Class: Dependabot::GitCommitChecker

Inherits:
Object
  • Object
show all
Extended by:
T::Sig
Includes:
GitCooldownDateResolver
Defined in:
lib/dependabot/git_commit_checker.rb

Overview

rubocop:disable Metrics/ClassLength

Constant Summary collapse

VERSION_REGEX =
/
  (?<version>
    (?<=^v)[0-9]+(?:\-[a-z0-9]+)?
    |
    [12][0-9]{3}(?:0[1-9]|1[0-2])(?:0[1-9]|[12][0-9]|3[01])
    |
    [0-9]+\.[0-9]+(?:\.[a-z0-9\-]+)*
  )$
/ix
VERSION_TAG_MATCH_PATTERN =

String pattern for matching version tags with optional prefixes (e.g., "v1.2.3" matches "1.2.3")

"(?:[^0-9\\.]|\\A)%s\\z"

Instance Method Summary collapse

Methods included from GitCooldownDateResolver

#cached_github_releases, #github_release_published_at, #normalize_tag_name, #resolve_candidate_date, #tag_creation_date

Constructor Details

#initialize(dependency:, credentials:, ignored_versions: [], raise_on_ignored: false, consider_version_branches_pinned: false, dependency_source_details: nil) ⇒ GitCommitChecker

Returns a new instance of GitCommitChecker.



53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
# File 'lib/dependabot/git_commit_checker.rb', line 53

def initialize(
  dependency:,
  credentials:,
  ignored_versions: [],
  raise_on_ignored: false,
  consider_version_branches_pinned: false,
  dependency_source_details: nil
)
  @dependency = dependency
  @credentials = credentials
  @ignored_versions = ignored_versions
  @raise_on_ignored = raise_on_ignored
  @consider_version_branches_pinned = consider_version_branches_pinned
  @dependency_source_details = dependency_source_details
end

Instance Method Details

#all_version_tagsObject



316
317
318
# File 'lib/dependabot/git_commit_checker.rb', line 316

def all_version_tags
  allowed_versions(local_tags, filter_by_prefix: false)
end

#allowed_version_refsObject



230
231
232
# File 'lib/dependabot/git_commit_checker.rb', line 230

def allowed_version_refs
  allowed_versions(local_refs)
end

#allowed_version_tagsObject



225
226
227
# File 'lib/dependabot/git_commit_checker.rb', line 225

def allowed_version_tags
  allowed_versions(local_tags)
end

#allowed_version_tags_with_release_datesObject



324
325
326
327
328
329
330
331
332
# File 'lib/dependabot/git_commit_checker.rb', line 324

def allowed_version_tags_with_release_dates
  allowed_version_tags.map do |tag|
    Dependabot::Package::PackageRelease.new(
      version: T.cast(version_from_tag(tag), Dependabot::Version),
      tag: tag.name,
      released_at: release_date_for_tag_name(tag.name)
    )
  end
end

#branch_or_ref_in_release?(version) ⇒ Boolean

Returns:

  • (Boolean)


150
151
152
# File 'lib/dependabot/git_commit_checker.rb', line 150

def branch_or_ref_in_release?(version)
  pinned_ref_in_release?(version) || branch_behind_release?(version)
end

#cooldown_credentialsObject



340
341
342
# File 'lib/dependabot/git_commit_checker.rb', line 340

def cooldown_credentials
  credentials
end

#cooldown_source_urlObject



335
336
337
# File 'lib/dependabot/git_commit_checker.rb', line 335

def cooldown_source_url
  dependency_source_details&.fetch(:url, nil)
end

#current_versionObject



235
236
237
238
239
# File 'lib/dependabot/git_commit_checker.rb', line 235

def current_version
  return unless dependency.version && version_tag?(T.must(dependency.version))

  version_from_ref(T.must(dependency.version))
end

#dependency_source_detailsObject



286
287
288
# File 'lib/dependabot/git_commit_checker.rb', line 286

def dependency_source_details
  @dependency_source_details || dependency.source_details(allowed_types: ["git"])
end

#filter_lower_versions(tags) ⇒ Object



242
243
244
245
246
247
248
249
250
251
252
# File 'lib/dependabot/git_commit_checker.rb', line 242

def filter_lower_versions(tags)
  return tags unless current_version

  versions = tags.map do |t|
    version_from_tag(t)
  end

  versions.select do |version|
    version > current_version
  end
end

#git_dependency?Boolean

Returns:

  • (Boolean)


70
71
72
73
74
# File 'lib/dependabot/git_commit_checker.rb', line 70

def git_dependency?
  return false if dependency_source_details.nil?

  dependency_source_details&.fetch(:type) == "git"
end

#git_repo_reachable?Boolean

Returns:

  • (Boolean)


278
279
280
281
282
283
# File 'lib/dependabot/git_commit_checker.rb', line 278

def git_repo_reachable?
  local_upload_pack
  true
rescue Dependabot::GitDependenciesNotReachable
  false
end

#head_commit_for_current_branchObject



155
156
157
158
159
160
161
162
# File 'lib/dependabot/git_commit_checker.rb', line 155

def head_commit_for_current_branch
  ref = ref_or_branch || "HEAD"

  sha = head_commit_for_local_branch(ref)
  return sha if pinned? || sha

  raise Dependabot::GitDependencyReferenceNotFound, dependency.name
end

#head_commit_for_local_branch(name) ⇒ Object



165
166
167
# File 'lib/dependabot/git_commit_checker.rb', line 165

def head_commit_for_local_branch(name)
  .head_commit_for_ref(name)
end

#head_commit_for_pinned_refObject



114
115
116
# File 'lib/dependabot/git_commit_checker.rb', line 114

def head_commit_for_pinned_ref
  .head_commit_for_ref_sha(T.must(ref))
end

#local_ref_for_latest_version_lower_precisionObject



177
178
179
180
181
# File 'lib/dependabot/git_commit_checker.rb', line 177

def local_ref_for_latest_version_lower_precision
  allowed_refs = local_tag_for_pinned_sha ? allowed_version_tags : allowed_version_refs

  max_local_tag_for_lower_precision(allowed_refs)
end

#local_ref_for_latest_version_matching_existing_precisionObject



170
171
172
173
174
# File 'lib/dependabot/git_commit_checker.rb', line 170

def local_ref_for_latest_version_matching_existing_precision
  allowed_refs = local_tag_for_pinned_sha ? allowed_version_tags : allowed_version_refs

  max_local_tag_for_current_precision(allowed_refs)
end

#local_tag_for_latest_version(cooldown_options = nil) ⇒ Object



191
192
193
194
195
196
# File 'lib/dependabot/git_commit_checker.rb', line 191

def local_tag_for_latest_version(cooldown_options = nil)
  filtered_tags = apply_cooldown(allowed_version_tags, cooldown_options)
  return nil if filtered_tags.nil?

  max_local_tag(filtered_tags)
end

#local_tag_for_pinned_shaObject



261
262
263
264
265
266
267
268
# File 'lib/dependabot/git_commit_checker.rb', line 261

def local_tag_for_pinned_sha
  return unless pinned_ref_looks_like_commit_sha?

  @local_tag_for_pinned_sha = T.let(
    most_specific_version_tag_for_sha(ref),
    T.nilable(String)
  )
end

#local_tag_for_pinned_version_ref(cooldown_options = nil) ⇒ Object



208
209
210
211
212
# File 'lib/dependabot/git_commit_checker.rb', line 208

def local_tag_for_pinned_version_ref(cooldown_options = nil)
  return nil unless pinned_ref_looks_like_version?

  local_tag_for_latest_version(cooldown_options)
end

#local_tags_for_allowed_versionsObject



220
221
222
# File 'lib/dependabot/git_commit_checker.rb', line 220

def local_tags_for_allowed_versions
  allowed_version_tags.filter_map { |t| to_local_tag(t) }
end

#local_tags_for_allowed_versions_matching_existing_precisionObject



215
216
217
# File 'lib/dependabot/git_commit_checker.rb', line 215

def local_tags_for_allowed_versions_matching_existing_precision
  select_matching_existing_precision(allowed_version_tags).filter_map { |t| to_local_tag(t) }
end

#max_local_tag(tags) ⇒ Object



309
310
311
312
313
# File 'lib/dependabot/git_commit_checker.rb', line 309

def max_local_tag(tags)
  max_version_tag = tags.max_by { |t| version_from_tag(t) }

  to_local_tag(max_version_tag)
end

#most_specific_tag_equivalent_to_pinned_refObject



255
256
257
258
# File 'lib/dependabot/git_commit_checker.rb', line 255

def most_specific_tag_equivalent_to_pinned_ref
  commit_sha = head_commit_for_local_branch(T.must(ref))
  most_specific_version_tag_for_sha(commit_sha)
end

#most_specific_version_tag_for_sha(commit_sha) ⇒ Object



296
297
298
299
300
301
# File 'lib/dependabot/git_commit_checker.rb', line 296

def most_specific_version_tag_for_sha(commit_sha)
  tags = local_tags_matching_sha(commit_sha)
  return if tags.empty?

  tags[-1]&.name
end

#most_specific_version_tags_for_sha(commit_sha) ⇒ Object



304
305
306
# File 'lib/dependabot/git_commit_checker.rb', line 304

def most_specific_version_tags_for_sha(commit_sha)
  local_tags_matching_sha(commit_sha).map(&:name)
end

#pinned?Boolean

Returns:

  • (Boolean)


78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
# File 'lib/dependabot/git_commit_checker.rb', line 78

def pinned?
  raise "Not a git dependency!" unless git_dependency?

  branch = dependency_source_details&.fetch(:branch)

  return false if ref.nil?
  return false if branch == ref
  return true if branch
  return true if dependency.version&.start_with?(T.must(ref))
  return true if ref_matches_tag?

  # Assume we're pinned unless the specified `ref` is actually a branch
  return true unless local_upload_pack&.match?(%r{ refs/heads/#{ref}$})

  # TODO: Research whether considering branches that look like versions pinned makes sense for all ecosystems
  @consider_version_branches_pinned && version_tag?(T.must(ref))
end

#pinned_ref_looks_like_commit_sha?Boolean

Returns:

  • (Boolean)


105
106
107
108
109
110
111
# File 'lib/dependabot/git_commit_checker.rb', line 105

def pinned_ref_looks_like_commit_sha?
  return false unless ref && ref_looks_like_commit_sha?(T.must(ref))

  return false unless pinned?

  .head_commit_for_ref(T.must(ref)).nil?
end

#pinned_ref_looks_like_version?Boolean

Returns:

  • (Boolean)


98
99
100
101
102
# File 'lib/dependabot/git_commit_checker.rb', line 98

def pinned_ref_looks_like_version?
  return false unless pinned?

  version_tag?(T.must(ref))
end

#ref_details(ref) ⇒ Object



127
128
129
130
131
132
133
134
135
136
137
# File 'lib/dependabot/git_commit_checker.rb', line 127

def ref_details(ref)
  T.must(
    T.let(
      GitMetadataFetcher.new(
        url: dependency.source_details&.fetch(:url, nil),
        credentials: credentials
      ).ref_details_for_pinned_ref(ref),
      T.nilable(Excon::Response)
    )
  )
end

#ref_details_for_pinned_refObject



140
141
142
# File 'lib/dependabot/git_commit_checker.rb', line 140

def ref_details_for_pinned_ref
  ref_details(ref_pinned)
end

#ref_looks_like_commit_sha?(ref) ⇒ Boolean

Returns:

  • (Boolean)


145
146
147
# File 'lib/dependabot/git_commit_checker.rb', line 145

def ref_looks_like_commit_sha?(ref)
  ref.match?(/^[0-9a-f]{6,40}$/)
end

#refs_for_tag_with_detailObject



291
292
293
# File 'lib/dependabot/git_commit_checker.rb', line 291

def refs_for_tag_with_detail
  .refs_for_tag_with_detail
end

#tagsObject



119
120
121
122
123
124
# File 'lib/dependabot/git_commit_checker.rb', line 119

def tags
  GitMetadataFetcher.new(
    url: dependency.source_details&.fetch(:url, nil),
    credentials: credentials
  ).tags
end

#version_for_pinned_shaObject



271
272
273
274
275
# File 'lib/dependabot/git_commit_checker.rb', line 271

def version_for_pinned_sha
  return unless local_tag_for_pinned_sha && version_class.correct?(local_tag_for_pinned_sha)

  version_class.new(local_tag_for_pinned_sha)
end