Module: Dependabot::HasSentryContext
- Extended by:
- T::Helpers, T::Sig
- Included in:
- SharedHelpers::HelperSubprocessFailed
- Defined in:
- lib/dependabot/errors.rb
Overview
Interface for error classes that provide Sentry context (e.g. fingerprint). Include this module in any error class that defines #sentry_context.
Instance Method Summary collapse
Instance Method Details
#sentry_context ⇒ Object
405 |
# File 'lib/dependabot/errors.rb', line 405 def sentry_context; end |